import os
from django.contrib.auth.decorators import login_required
from django.shortcuts import render, redirect
from django.contrib.auth.forms import UserCreationForm
from django.contrib.auth import login as auth_login
from .models import Incoming, Storage

from django.shortcuts import get_object_or_404, redirect

from django.http import HttpResponse, HttpResponseForbidden

from .models import Order, OrderArchive

from rest_framework.response import Response
from rest_framework.decorators import api_view
from .models import Order
from .serializers import OrderSerializer





from django.http import JsonResponse, Http404
from django.views.decorators.csrf import csrf_exempt
from django.conf import settings
from .models import Order, APIKey

from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.permissions import IsAuthenticated
from django.conf import settings
import os

from django.http import HttpResponse, Http404
from django.conf import settings
import os
from urllib.parse import quote

def download_file(request, file_name):
    # Проверка API ключа
    api_key = request.headers.get('Authorization')
    if not api_key or api_key != f'Api-Key {settings.API_KEY}':
        return HttpResponse('Unauthorized', status=403)
    
    # Путь к файлу
    file_path = os.path.join(settings.MEDIA_ROOT, file_name.replace('media/', ''))
    print('PATH PATH PATH:', file_path)
    if not os.path.exists(file_path):
        raise Http404("File not found")

    # Отправка файла
    with open(file_path, 'rb') as file:
        response = HttpResponse(file.read(), content_type='application/octet-stream')
        response['Content-Disposition'] = f'attachment; filename="{quote(os.path.basename(file_name))}"'
        return response
    

# class DownloadFileView(APIView):
#     permission_classes = [IsAuthenticated]

#     def get(self, request, file_path):
#         full_path = os.path.join(settings.MEDIA_ROOT, file_path)
        
#         if not os.path.exists(full_path):
#             return Response({'error': 'File does not exist'}, status=404)
        
#         with open(full_path, 'rb') as file:
#             response = Response(file.read(), content_type='application/octet-stream')
#             response['Content-Disposition'] = f'attachment; filename={os.path.basename(full_path)}'
#             return response

@csrf_exempt
def order_list_api(request):
    # Проверка ключа API
    api_key = request.headers.get('Authorization')
    if api_key != f"Api-Key {settings.API_KEY}":
        return JsonResponse({'error': 'Unauthorized'}, status=403)

    if request.method == 'GET':
        orders = list(Order.objects.values())
        return JsonResponse(orders, safe=False)
    return JsonResponse({'error': 'Method not allowed'}, status=405)







@csrf_exempt
@api_view(['POST'])
def mark_order_as_processed(request, order_id):
    # Проверка API ключа
    api_key = request.headers.get('Authorization')
    if api_key != f'Api-Key {settings.API_KEY}':
        return JsonResponse({'error': 'Unauthorized'}, status=403)
    
    # Получение статуса из запроса
    status = request.data.get('status')
    if status not in ['success', 'failed']:
        return JsonResponse({'error': 'Invalid status'}, status=400)
    
    try:
        order = Order.objects.get(id=order_id)
    except Order.DoesNotExist:
        return JsonResponse({'error': 'Order not found'}, status=404)

    # Создание записи в Incoming
    incoming_record = Incoming(order=order, job_state=status)
    incoming_record.save()
    
    # Обновление состояния заказа
    order.order_state = 'done' if status == 'success' else 'canceled'
    order.save()
    
    return JsonResponse({'message': 'Order marked as processed'}, status=200)













@api_view(['GET'])
def order_list(request):
    orders = Order.objects.all()  # Получаем все заказы
    serializer = OrderSerializer(orders, many=True)  # Сериализуем данные
    return Response(serializer.data)  # Возвращаем JSON-ответ






# Create your views here.
@login_required
def home(request):
    is_manager = request.user.groups.filter(name='manager').exists()
    return render(request, 'mainapp/home.html', {'is_manager': is_manager})
    # return render(request, 'mainapp/home.html')

@login_required
def make_order(request):
    return render(request, 'mainapp/make_order.html')




@login_required
def view_archive(request):
    user_archive_orders = OrderArchive.objects.filter(user=request.user)

    statuses = ['done', 'canceled']
    archive_orders_by_status = {status: user_archive_orders.filter(order_state=status) for status in statuses}

    archive_orders_status_list = []
    for status in statuses:
        archive_orders = archive_orders_by_status[status]
        for order in archive_orders:
            if order.file_path:
                order.file_basename = os.path.basename(order.file_path)
            else:
                order.file_basename = []
        archive_orders_status_list.append((status, archive_orders))

    return render(request, 'mainapp/view_archive.html', {'archive_orders_status_list': archive_orders_status_list})



@login_required
def archive_order(request, order_id):
    order = get_object_or_404(Order, id=order_id)

    if order.order_state in ['done', 'canceled']:
        OrderArchive.objects.create(
            user=order.user,
            order_datetime=order.order_datetime,
            order_parameters=order.order_parameters,
            file_path=order.file_path,
            order_state=order.order_state,
        )
        order.delete()

    return redirect('view_orders')



@login_required
def view_orders(request):
    user_orders = Order.objects.filter(user=request.user)

    statuses = ['wait_confirmation', 'in_work', 'ready', 'done', 'canceled']
    orders_by_status = {status: user_orders.filter(order_state=status) for status in statuses}

    orders_status_list = []
    for status in statuses:
        orders = orders_by_status[status]
        for order in orders:
            # Извлекаем только basename из файловых путей
            if order.file_path:
                order.file_basename = os.path.basename(order.file_path) 
                
            else:
                order.file_basename = []
        orders_status_list.append((status, orders))

    return render(request, 'mainapp/view_orders.html', {'orders_status_list': orders_status_list})

from .forms import OrderForm 

@login_required
def create_order(request):
    if request.method == 'POST':
        form = OrderForm(request.POST, request.FILES, user=request.user)
        if form.is_valid():
            order = form.save(commit=False)
            order.user = request.user

            file_choice = form.cleaned_data.get('file_choice')

            if file_choice == OrderForm.CHOICE_FILE_FROM_STORAGE:
                storage_item = form.cleaned_data.get('storage_item')
                if storage_item:
                    order.file_path = storage_item.file_path
            elif file_choice == OrderForm.CHOICE_FILE_UPLOAD:
                uploaded_file = request.FILES.get('upload_file')
                if uploaded_file:
                    file_path = os.path.join('media', uploaded_file.name)
                    with open(file_path, 'wb+') as destination:
                        for chunk in uploaded_file.chunks():
                            destination.write(chunk)
                    order.file_path = file_path

            order.save()
            return redirect('view_orders')
    else:
        form = OrderForm(user=request.user)
    
    return render(request, 'mainapp/create_order.html', {'form': form})


@login_required
def storage(request):
    items = Storage.objects.filter(user=request.user)
    return render(request, 'mainapp/storage.html', {'items': items})


from django.http import FileResponse

@login_required
def download_item(request, item_id):
    item = get_object_or_404(Storage, id=item_id, user=request.user)
    
    # Проверяем, существует ли файл
    if os.path.exists(item.file_path):
        # Отправляем файл пользователю для скачивания
        return FileResponse(open(item.file_path, 'rb'), as_attachment=True, filename=os.path.basename(item.file_path))
    else:
        # Если файла нет, можно вернуть ошибку или редирект
        return redirect('storage')

@login_required
def delete_item(request, item_id):
    item = get_object_or_404(Storage, id=item_id, user=request.user)  # Проверка, что элемент принадлежит текущему пользователю
    if item.user == request.user:
        if os.path.isfile(item.file_path):
            os.remove(item.file_path)
        item.delete()
        return redirect('storage')  # После удаления перенаправляем пользователя на страницу хранилища
    else:
        return HttpResponseForbidden("Вы не можете удалить этот элемент.")
    


from .forms import UploadFileForm
import os

@login_required
def upload_file(request):
    if request.method == 'POST':
        form = UploadFileForm(request.POST, request.FILES)
        if form.is_valid():
            storage_item = form.save(commit=False)
            storage_item.user = request.user

            # Определяем путь к папке пользователя
            user_directory = os.path.join('media', request.user.username)

            # Создаем папку, если она не существует
            if not os.path.exists(user_directory):
                os.makedirs(user_directory)

            # Сохранение файла
            uploaded_file = request.FILES['file']
            file_name = uploaded_file.name
            file_path = os.path.join(user_directory, file_name)

            # Сохранение файла на диск
            with open(file_path, 'wb+') as destination:
                for chunk in uploaded_file.chunks():
                    destination.write(chunk)

            # Сохранение пути в базу данных
            storage_item.file_path = file_path
            storage_item.save()

            return redirect('storage')  # Перенаправление на страницу хранилища
    else:
        form = UploadFileForm()

    return render(request, 'mainapp/upload_file.html', {'form': form})

@login_required
def profile(request):
    return render(request, 'mainapp/profile.html')

def main_page(request):
    return render(request, 'mainapp/main_page.html')


def register(request):
    if request.method == 'POST':
        form = UserCreationForm(request.POST)
        if form.is_valid():
            user = form.save()
            auth_login(request, user)
            return redirect('home')  # Перенаправление после регистрации
    else:
        form = UserCreationForm()
    return render(request, 'registration/register.html', {'form': form})


@login_required
def manager_dashboard(request):
    if request.user.groups.filter(name='manager').exists():
        if request.method == 'POST':
            order_id = request.POST.get('order_id')
            new_status = request.POST.get('order_state')
            if order_id and new_status:
                order = get_object_or_404(Order, id=order_id)
                order.order_state = new_status
                order.save()
                return redirect('manager_dashboard')  # Перенаправляем обратно на панель менеджера

        # Получаем параметр сортировки из запроса (по умолчанию сортируем по дате заказа)
        sort_by = request.GET.get('sort', 'order_datetime')
        
        # Проверяем, что параметр сортировки валиден
        if sort_by not in ['order_datetime', 'order_state']:
            sort_by = 'order_datetime'
        
        # Получаем все заказы для отображения
        orders = Order.objects.all().order_by(sort_by)
        for order in orders:
            if order.file_path:
                order.file_basename = os.path.basename(order.file_path)
            else:
                order.file_basename = None
        return render(request, 'mainapp/manager_dashboard.html', {'orders': orders})
    else:
        return redirect('home')
    



def serve_file(request, file_name):
    file_path = os.path.join(settings.MEDIA_ROOT, file_name)
    print(file_path)
    if not os.path.exists(file_path):
        raise Http404("File not found")
    return FileResponse(open(file_path, 'rb'))